#ifndef WORLDB_H
#define WORLDB_H

#include "btBulletDynamicsCommon.h"
// #include "LinearMath/btIDebugDraw.h"
// #include "../../gl/gldebugdrawer.h"
// #include <sstream>
#include <iomanip>
// #include <vector>
// #include <string>
// #include <cstring> 
#include "../../utils/timer.h"
#include "../../utils/dir.h"
#include "../../utils/file.h"
#include "../../utils/parser.h"
#include "../../utils/randgen.h"
#include "../../utils/settings.h"
#include "../../utils/raycast.h"
#include "../../utils/mousepicker.h"
#include "../../utils/statsbuffer.h"
#include "../../utils/critterselection.h"
#include "../../utils/logbuffer.h"

#include <cmath>

#include "../gui/textverbosemessage.h"

#include "circuit8.h"
#include "food.h"
#include "pill.h"
#include "particle.h"
#include "wall.h"
#include "critterb.h"
#include "camera.h"

using namespace std;



class WorldB
{
	public:
		WorldB();
		virtual			~WorldB();
		virtual void		init();
		virtual void		maybescroll();
		virtual void		process();
		virtual btVector3	findPosition();
		virtual			void makeFloor();
		float			insertHight;
		void			getGeneralStats();
		void			killHalf();
		void			expireFood();
		void			expireCritters();
		void			autoinsertFood();
		void			autosaveCritters();
		void			autoexchangeCritters();
		void			autoinsertCritters();
		void			eat( CritterB* c );
		void			procreate( CritterB* c );
		void			makeDefaultFloor();
		
		Settings*		settings;
		Critterselection	*critterselection;
		Statsbuffer*		statsBuffer;
		RandGen*		randgen;
		Mousepicker*		mousepicker;
		Camera camera;
		void resetCamera();

		//SpuGatheringCollisionDispatcher*	m_dispatcher;
		btCollisionDispatcher*	m_dispatcher;
		btBroadphaseInterface*	m_broadphase;
		btDefaultCollisionConfiguration* m_collisionConfiguration;
		btDynamicsWorld*	m_dynamicsWorld;
		btConstraintSolver*	m_solver;
		btAlignedObjectArray<btCollisionShape*>	m_collisionShapes;


		vector<CritterB*>		cRitters;
		vector<Food*>			food;
		vector<Pill*>			pill;
		vector<Particle*>		Particles;
		unsigned int			pillidx;
		unsigned int			pillsinworld;
		vector<Wall*>			walls;
//		vector<CritterB>		offspring_queue;
//		vector<CritterB>		zombies;
//		typedef float			zombie_species[4];
//		vector<unsigned int>			zombie_species_list; // uses hashes
//		static const bool		insert_zombies;
//		static const unsigned int	offspring_queue_length = 8;
//		static const unsigned int	zombie_queue_length = 8;

//		static const unsigned int	auto_excit_agon_every = 1000;
//		static const unsigned int	auto_excit_agon_amt = 609;
		unsigned int			auto_excit_agon_ctr;
//		static const unsigned int	auto_excit_antag_every = 0;
//		static const unsigned int 	auto_excit_antag_amt = 0;
		unsigned int			auto_excit_antag_ctr;
//		static const unsigned int	auto_inhib_agon_every = 0;
//		static const unsigned int	auto_inhib_agon_amt = 0;
		unsigned int			auto_inhib_agon_ctr;
//		static const unsigned int	auto_inhib_antag_every = 1071;
//		static const unsigned int	auto_inhib_antag_amt = 201;
		unsigned int			auto_inhib_antag_ctr;
//		static const unsigned int	auto_potentiator_every = 1250;
//		static const unsigned int	auto_potentiator_amt = 200;
		unsigned int			auto_potentiator_ctr;
//		static const bool		decrement_autodrug_amts = true;

		unsigned long		currentCritterID;

		float			freeEnergy;
		//float			freeEnergyInfo;

		void			drawWithGrid();
		void			drawWithoutFaces();
		
		void			drawWithinCritterSight(CritterB *c);

		void			startfoodamount(unsigned int amount);

		int			insertRandomFood(int amount, float energy);
		int			insertNextPill(int amount);

		virtual void		insertCritter();
// 		void			positionCritterB(unsigned int cid);
		void			saveAllCritters();
//		void			loadMovie(bool frompanel);
		void			loadMovie();
		void			loadMovieForReal();

		virtual void		loadAllCritters();

		void			killHalfOfCritters();

		void			pickBody(const int& x, const int& y);
		void			selectBody(const int& x, const int& y);
		void			castMouseRay();
		void			movePickedBodyTo();
		void			movePickedBodyFrom();
		float			autosaveCounter;
		float			autoexchangeCounter;

		// vision
		unsigned char		*retina;
		unsigned int		items;

		const unsigned int*	retinasperrow;
		const unsigned int*	critter_retinasize;
		const unsigned int*	critter_maxenergy;

		void			checkCollisions( CritterB* c );

		void grabVision();

		void renderVision();

		void			calcMouseDirection();
		void			moveInMouseDirection(bool towards);

		Dir			dirH;
		Parser*			parseH;
		File			fileH;

		string			loaddir;
		string			scrndir;


		bool			mouseRayHit;
		Entity*			mouseRayHitEntity;
		Circuit8		circuit8;

		void getGroupScreen(CritterB* c);
		void alterGroupScreen(CritterB* c);
		void newParticles(btDefaultMotionState* myMotionState, float r, float g, float b, float a, unsigned int num, signed int minheight, signed int maxheight, unsigned int maxspeed, unsigned int xzspread, unsigned int type, float foodlevel, unsigned int drugtype, float druglevel);
		void updateParticles();
		void killParticle(unsigned int i);


		// camera operations (needed for commands)
		void 			camera_moveup();
		void 			camera_movedown();
		void 			camera_moveforward();
		void 			camera_movebackward();
		void 			camera_moveleft();
		void 			camera_moveright();
		void 			camera_lookup();
		void 			camera_lookdown();
		void 			camera_lookleft();
		void 			camera_lookright();
		void 			camera_rollleft();
		void 			camera_rollright();

		// pointers to parents (evolution) mousepos
		int* mousex;
		int* mousey;

		void			tripfirst();
		void			tripfirst2();
		void			cyclescreenmodes(bool dir);
		void			clearnovote();
		void			tripall1();
		void			tripall2();
		void			tripall3();
		void			tripall4();
		void			tripall5();
		void			tripall1(unsigned int amt);
		void			tripall2(unsigned int amt);
		void			tripall3(unsigned int amt);
		void			tripall4(unsigned int amt);
		void			tripall5(unsigned int amt);
		void			untripall1();
		void			untripall2();
		void			untripall3();
		void			untripall4();
		void			untripall5();
		void			autoDrug();
		
		void 			tripthis1();
		void			tripthis2();
		void			tripthis3();
		void			tripthis4();
		void			tripthis5();
		void 			untripthis1();
		void			untripthis2();
		void			untripthis3();
		void			untripthis4();
		void			untripthis5();
		void			feedthis1();
		void			togglelockblink1();
		void			togglelockblink2();
		void			togglelockdruglevels();

		unsigned int		hashSpeciesColour(float r, float g, float b, float a);

		// voting
		unsigned int			novotesfor;
		unsigned int			downvotes;
		unsigned int			upvotes;
		unsigned int			leftvotes;
		unsigned int			rightvotes;
		unsigned int			firevotes;
		unsigned int			erifvotes;
		unsigned int			pausevotes;
		unsigned int			dispersevotes;
		unsigned int			disrepsevotes;
		unsigned int			mostvotes;

		unsigned int			scrn_delay_ctr;

		unsigned int				mostdrugs;

		static const bool		evolve_set_to_grey = true; // temp, put in settings (actually sets for gradient, maybe
									    // move this to set-for-monochrome-gradient or something and
									    // put old evolve_set_to_grey behaviour back in for this
									   
		static const bool		just_kill_oldest = false;    // temp, put in settings
		static const float		clipping_floor = -20.0f;   // temp, put in settings (for falling world)
		static const unsigned int	maxcritters = 200;           // temp, put in settings (does nothing yet)

		static const unsigned int	particledensity = 2; // make this a cvar
		static const unsigned int	maxparticles = 400;  // cvar this
		static const unsigned int	particleUpdateFreq = 4;  // cvar this
		static const unsigned int	shootinterval = 1000; // make cvar
		
		unsigned int			particleCtr;
		btDefaultMotionState		eddms;
		unsigned int			evildeadctr;


	private:
		Raycast*		raycast;

		castResult		mouseRay;
		btVector3		mouseRayTo;

		unsigned int		insertCritterCounter;
		unsigned int		showscrnshotctr;
// 		GLDebugDrawer debugDrawer;

		// FIXME give these a better place.  We also want to know where to save profiles to
		string			homedir;
		string			progdir;
		string			savedir;

		// methods
		inline void		removeCritter(unsigned int cid);
		inline void		createDirs();

		// Settings pointers.. performance
		const unsigned int*	critter_maxlifetime;
		const unsigned int*	critter_autosaveinterval;
		const unsigned int*	critter_autoexchangeinterval;
		const unsigned int*	critter_killhalfat;
		const unsigned int*	critter_sightrange;
		const unsigned int*	critter_raycastvision;

		const unsigned int*	food_maxlifetime;
		const unsigned int*	food_maxenergy;
		const unsigned int*	energy;
		
		const unsigned int*	pill_maxlifetime;
		const unsigned int*	pill_energylevel;
		const unsigned int*	pill_maxtrip;

		const unsigned int*	condensed_colourmotors;

		// vision opts
		unsigned int picwidth;
		
		btManifoldArray   manifoldArray;

};

#endif
